<?php
/*
Plugin Name: Post Password Token
Plugin URI: http://top-frog.com/projects/post-password-token/
Description: Allow tokens to be supplied in the URL to negate the post_password requirement. Mimics the Guest Pass functionality on Flickr. <a href="options-general.php?page=post-password-token.php">Configure plugin options</a>
Version: 1.0.2
Author: shawnparker, gordonbrander
Author URI: http://top-frog.com
*/

// Copyright (c) 2009 Shawn Parker, Gordon Brander. All rights reserved.
//
// Released under the GPL license
// http://www.opensource.org/licenses/gpl-license.php
//
// **********************************************************************
// This program is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
// **********************************************************************
	
	define('PPT_OPTION','ppt-token-options');
	define('PPT_COOKIE','wp-post-token_');
	define('PPT_PLUGIN_URL', plugins_url().'/'.basename(__FILE__,'.php').'/');
	define('PPT_VER', '1.0.2');

// Front end Auth

	/**
	 * check the post_password token at template redirect
	 *
	 * @return void
	 */
	function ppt_template_redirect() {
		global $wp_query;
		if((is_single() || is_page()) && isset($wp_query->post->post_password) && isset($_GET['ppt'])) {
			if(ppt_cookie_match($wp_query->post,$_GET['ppt'])) {
				ppt_set_cookie($wp_query->post->post_password);
			}
		}
	}
	add_action('template_redirect','ppt_template_redirect');

	/**
	 * Attempt a token match
	 *
	 * @param object $post 
	 * @param string $token 
	 * @return bool
	 */
	function ppt_cookie_match($post,$token) {
		if(!isset($_COOKIE[PPT_COOKIE.COOKIEHASH]) || $_COOKIE[PPT_COOKIE.COOKIEHASH] != ppt_make_token($post)) {
			return ppt_make_token($post) == $token;
		}
		return false;
	}

	/**
	 * Set the cookie 
	 * Functionality duplicated from WordPress' post password submit in wp-pass.php
	 *
	 * @param string $post_password 
	 * @return void
	 */
	function ppt_set_cookie($post_password) {
		global $token;
		setcookie(PPT_COOKIE.COOKIEHASH, $token, null, COOKIEPATH);
		setcookie('wp-postpass_' . COOKIEHASH, $post_password, time() + 864000, COOKIEPATH);
		wp_redirect('http://'.$_SERVER['SERVER_NAME'].$_SERVER['REQUEST_URI']);
		exit;
	}

// Admin

	/**
	 * Add the meta box to the post/page edit meta boxes
	 *
	 * @return void
	 */
	function ppt_add_meta_box() {
		add_meta_box('ppt-token', __('Post Password Token'), 'ppt_meta_box', ppt_meta_get_type(), 'normal', 'high');
	}
	add_action('admin_head','ppt_add_meta_box');

	/**
	 * Output informational meta box
	 *
	 * @return void
	 */
	function ppt_meta_box() {
		global $post;
		
		// if no password set message and exit quickly
		if(!isset($post->post_password) || empty($post->post_password)) { 
			echo '<p>There are no password token URLs for this post. Set a post password to enable the password token.</p>';
			return;
		}

		// build link
		$url = ppt_make_permalink($post);
		$html = '
			<dl>
				<dd><p><a href="'.$url.'">'.$url.'</a></p></dd>
				<dt><p class="ppt-blurb">Copy and share this secret <em>Password Token URL</em> to allow readers to see the content of this post. <a href="options-general.php?page=post-password-token">Learn more &raquo;</a></p></dt>
			</dl>
			';
		echo $html;
	}
	
	/**
	 * determine which type we're working on
	 */
	function ppt_meta_get_type() {
		global $pagenow;
		switch(true) {
			case in_array($pagenow, array('post.php','post-new.php')):
				return 'post';
				break;
			case in_array( $pagenow, array('page.php','page-new.php')):
				return 'page';
				break;
			default:
				return false;
		}
	}
	
// Admin Page

	/**
	 * Admin init, preppity prep-prep
	 *
	 * @return void
	 */
	function ppt_admin_init() {
		if(strtolower($_SERVER['REQUEST_METHOD']) == 'post' && isset($_POST['ppt-save-options'])) {
			ppt_admin_save();
		}
		add_contextual_help('settings_page_post-password-token', ppt_admin_contextual_help());
	}
	add_action('admin_init','ppt_admin_init',10);

	/**
	 * Contextual Help Menu Item
	 * 
	 * @return string html
	 */
	function ppt_admin_contextual_help() {
		return '<p>Help for the Post Password Token can be found through our <a href="http://top-frog.com/projects/post-password-token/">Project Page</a>.</p>';
	}

	/**
	 * Add the admin menu item
	 *
	 * @return void
	 */
	function ppt_admin_menu_item(){
		add_submenu_page('options-general.php',__('Post Password Token'),__('Post Password Token'),10,basename(__FILE__),'ppt_admin_page');
	}
	add_action('admin_menu','ppt_admin_menu_item');
	
	/**
	 * Save the options
	 *
	 * @return void
	 */
	function ppt_admin_save($salt=null) {
		if($salt == null && isset($_POST['ppt_salt'])) {
			$salt = strval($_POST['ppt_salt']);
			$redirect=true;
			if(!get_magic_quotes_gpc()) {
				$salt = stripslashes($salt);
			}
		}

		if( get_option(PPT_OPTION) === false ) {
			add_option(PPT_OPTION, $salt, null, 'no');
		}
		else {
			update_option(PPT_OPTION, $salt);
		}
		if($redirect) {
			wp_redirect('http://'.$_SERVER['SERVER_NAME'].$_SERVER['REQUEST_URI']);
			exit;
		}
	}
	
	/**
	 * Create a default salt
	 *
	 * @return string
	 */
	function ppt_create_salt() {
		$salt = substr( crypt(md5(time()) ) ,0,32);
		return $salt;
	}
	
	/**
	 * Output admin scripts and styles
	 *
	 * @return void
	 */
	if (is_admin()) {
		global $pagenow;
		if($_GET['page'] == basename(__FILE__)) {
			wp_enqueue_script('ppt-admin',PPT_PLUGIN_URL.'js/ppt-admin.js',array('jquery'),PPT_VER);
			wp_enqueue_style('ppt-admin',PPT_PLUGIN_URL.'css/ppt-admin.css',array(),PPT_VER,'screen');
		}
	}
	
	/**
	 * Output the admin page
	 *
	 * @return void
	 */
	function ppt_admin_page() {
		$salt = get_option(PPT_OPTION,'');
		if(empty($salt)) {
			$salt = ppt_create_salt();
			ppt_admin_save($salt);
		}

		// UI
		echo '
			<div id="ppt-wrap" class="wrap">
				';
		if(function_exists('screen_icon')) { 
			screen_icon(); 
		}
		echo '
				<h2>'.__('Post Password Token').'</h2>
				<p>Issue secret Password token urls that allow readers to access password-protected posts without having to type in a password. This is similar to Flickr&rsquo;s "Guest Pass" functionality.</p>

				<p>Accessing a password-protected post by its url will still show the standard password dialog, but if a reader accesses a password-protected post by its <strong>secret Password Token url</strong>, they will be automatically authenticated and be able to see the full content. Accessing the post by its secret url will also set an authentication cookie for the user that lasts for 10 days.</p>
				
				<hr class="ppt-hr" />
				
				<h3>Password Salt</h3>
				<form id="token-profile" name="token-profile" method="post" >
					<p class="help">A "salt" is a secret code key that is used when creating tokens, making them more secure. We recommend that you <strong>set this once and leave&nbsp;it</strong>.</p>
					<div class="ppt-danger-box">
						<p>
							<label for="ppt_salt">Add some salt</label>
							<input type="text" size="50" id="ppt_salt" name="ppt_salt" class="lock" value="'.(isset($salt) ? htmlspecialchars($salt) : '').'" />
						</p>
						<div id="token-profile-submit">
							<p><strong>Warning</strong>: changing the salt will modify all <em>Password Token URLs</em> site-wide: readers will no longer be able to use old <em>Password Token URLs</em> to view protected content.</p>
							<p>
								<button type="submit" class="button-primary" id="ppt-save-options" name="ppt-save-options">Save salt, change tokens for all posts</button> <span class="hide-if-no-js">or <a class="lock-cancel" href="#ppt_salt">cancel</a></span>
							</p>
						</div><!--/token-profile-submit-->
					</div>
				</form>
				
				<hr class="ppt-hr" />
				
				<h3>How do I revoke the <em>Password Token URL</em> for a post?</h3>
				
				<p>Just change the password on the post. This will change the token, meaning the old link will no longer authenticate readers.</p>
				
				<p>If you need to revoke all <em>Password Token URL</em>s everywhere (the nuclear option), you can change the Password Salt above. This will create new tokens for all protected posts and invalidate all old tokens.</p>
				
				<h3>A note about Caching</h3>
				<p>If your site uses caching (like WP-Super-Cache) these pages will be cached. While this isn&rsquo;t really a security threat of galactic proportions (not any more so that what this plugin does already and, really, you wouldn&rsquo;t be using WordPress if national security was at stake) you can stop the pages from being cached if you prefer by adding <code>ppt=</code> to the caching exclusions list in WP-Super-Cache to keep the pages from being cached.</p>
				
				<div id="donate">
					<h3>Please Donate</h3>
					<p>Donations buy donuts. Donuts help keep us motivated. When we&rsquo;re motivated we make and update plugins. Please help keep us motivated to make more useful plugins.</p>
					<div id="paypal">
						<form action="https://www.paypal.com/cgi-bin/webscr" method="post">
							<input type="hidden" name="cmd" value="_s-xclick">
							<input type="hidden" name="hosted_button_id" value="6908957">
							<input type="image" src="https://www.paypal.com/en_US/i/btn/btn_donateCC_LG.gif" border="0" name="submit" alt="PayPal - The safer, easier way to pay online!">
							<img alt="" border="0" src="https://www.paypal.com/en_US/i/scr/pixel.gif" width="1" height="1">
						</form>
					</div>
				</div>
			</div>
		';
	}
	
// Core

	/**
	 * Build our custom permalink with token
	 *
	 * @param object $post 
	 * @return void
	 */
	function ppt_make_permalink($post) {
		$permalinks = get_option('permalink_structure');
		return get_permalink($post->ID).('' != $permalinks ? '?ppt=' : '&ppt=').ppt_make_token($post);
	}

	/**
	 * Make an access hash
	 * Currently as simple as md5($post_name.$post_password)
	 *
	 * @param object $post 
	 * @return string
	 */
	function ppt_make_token($post) {
		global $token;
		if(is_null($token)) {
			$salt = get_option(PPT_OPTION,'');
			if(empty($salt)) {
				$salt = ppt_create_salt();
				ppt_admin_save($salt);
			}
			$token = md5($salt.$post->post_name.$post->post_password);
		}
		return $token;
	}

?>